#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
# Description:

from requests.auth import HTTPBasicAuth
import json
import requests
import base64
import time
import os
import configparser
import sys

API_URL = 'https://api.github.com{}'
ACCEPT = 'application/vnd.github.v3+json'

class BaseObject(object):
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def to_dict(self):
        return json.loads(self.to_json())

    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True)

    def __str__(self):
        return self.to_json()

class Content(BaseObject):
    pass

class Author():
    def __init__(self, name, email):
        self.name = name
        self.email = email

class Response(BaseObject):
    def __init__(self):
        self.contents = []
    pass

class Github():
    def __init__(self, user, password):
        self.user = user
        self.password = password

    def set_author(self, name, email):
        self.author = Author(name, email)

    def get_file(self, owner, repository, path, ref='master'):
        '''
        获取文件
        https://developer.github.com/v3/repos/contents/#get-contents
        '''

        url = '/repos/{}/{}/contents/{}'.format(owner, repository, path)
        self._get(url, ref=ref)
        is_success = self.response.status_code == 200
        return is_success, self.response.json()

    def create_from_file(self, owner, repository, filepath, path=None, sha=None):
        '''
        创建文件
        https://developer.github.com/v3/repos/contents/#create-a-file
        '''
        if not path:
            path = os.path.basename(filepath)

        content = ""
        with open(filepath, 'rb') as f:
            byte_data = f.read()
            content = base64.b64encode(byte_data).decode()

        url = '/repos/{}/{}/contents/{}'.format(owner, repository, path)
        data = {
            "message": "Create file {}".format(path),
            "author": {
                "name": self.author.name,
                "email": self.author.email
                },
            "content": content
            }
        if sha:
            data['sha'] = sha
        self._put(url, **data)
        return self.response.status_code, self.response.json()

    def create_repository(self, name, description="", homepage="",
            private=False, has_issues = True, has_projects = True,
            has_wiki = True, **kw
        ):
        '''
        创建仓库 https://developer.github.com/v3/repos/#create
        '''
        all_args = locals()
        url = '/user/repos'
        all_args.pop('self')
        self._post(url, **all_args)
        success = self.response.status_code == 201
        return success, self.response.json()

    def create_organization_repository(self, organization, name,
            description="", homepage="", private=False, has_issues = True,
            has_projects = True, has_wiki = True, **kw
        ):
        '''
        创建组织仓库 https://developer.github.com/v3/repos/#create
        '''
        all_args = locals()
        url = '/orgs/{}/repos'.format(organization)
        all_args.pop('self')
        self._post(url, **all_args)
        success = self.response.status_code == 201
        return success, self.response.json()

    def _get(self, url, **kw):
        res = requests.get(
            API_URL.format(url),
            params=kw,
            headers={"Accept": ACCEPT},
            auth=HTTPBasicAuth(self.user, self.password)
        )
        #  print(res.status_code)
        #  print(res.json())
        self.response = res

    def _put(self, url, **kw):
        res = requests.put(
            API_URL.format(url),
            json=kw,
            headers={"Accept": ACCEPT},
            auth=HTTPBasicAuth(self.user, self.password)
        )
        #  print(res.status_code)
        #  print(res.json())
        self.response = res

    def _post(self, url, **kw):
        res = requests.post(
            API_URL.format(url),
            json=kw,
            headers={"Accept": ACCEPT},
            auth=HTTPBasicAuth(self.user, self.password)
        )
        self.response = res
        #  print(res.status_code)
        #  print(res.json())

if __name__ == "__main__":
    config_path = '{}/.config/gos/config'.format(os.getenv("HOME"))
    credential_path = '{}/.config/gos/credentials'.format(os.getenv("HOME"))

    credential = configparser.ConfigParser()
    credential.read(credential_path)
    default_cred = credential['default']

    g = Github(default_cred['user'], default_cred['password'])

    config = configparser.ConfigParser()
    config.read(config_path)
    author = config['user']
    g.set_author(author['name'], author['email'])

    owner = config['repository']['owner']
    repo = config['repository']['repo']
    #  g.create_from_file('image', '/Users/wxnacy/Downloads/mac-font2.png',
            #  'common/{}'.format(time.time()))

    #  g.create_repository('test2')
    #  g.create_organization_repository(, 'test3')
    g.get_file('wxnacy', 'image', 'push')




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
# Description:

import click
from goss import Github
from goss import __version__
from goss.app import utils
import configparser
import os
import pyperclip

GOSS_CONFIG_HOME = '{}/.config/goss'.format(os.getenv("HOME"))
GOSS_CONFIG_PATH = '{}/config'.format(GOSS_CONFIG_HOME)
GOSS_CREDENTIAL_PATH = '{}/credentials'.format(GOSS_CONFIG_HOME)


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(__version__.__version__)
    ctx.exit()

@click.group()
@click.option('--version', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True)
@click.pass_context
def run(ctx):
    '''
    Github Object Storage
    '''
    if not os.path.exists(GOSS_CONFIG_HOME):
        os.mkdir(GOSS_CONFIG_HOME)
    if os.path.exists(GOSS_CREDENTIAL_PATH):
        cred = configparser.ConfigParser()
        cred.read(GOSS_CREDENTIAL_PATH)
        secs = cred.sections()
        if 'default' in secs:
            default_cred = cred['default']
            user = default_cred['user']
            password = default_cred['password']
            ctx.obj = Github(user, password)

def init_credentials_path():
    if not os.path.exists(GOSS_CONFIG_HOME):
        os.mkdir(GOSS_CONFIG_HOME)

    #  if not os.path.exists(GOSS_CREDENTIAL_PATH):
        #  with open(GOSS_CREDENTIAL_PATH, '+w') as f:
            #  f.flush()
            #  f.close()


@run.command()
@click.option('--user', '-u', prompt='Your Github user', help = 'Your Github user')
@click.option('--password', '-p', prompt='Your Github password',
        help = 'Your Github password',hide_input=True)
@click.pass_context
def credentials(ctx, user, password):
    g = ctx.obj
    confirm = click.style('The credentials is exist. Do you want replace it?', fg='red')
    if g and not click.confirm(confirm):
        return
    conf = configparser.ConfigParser()
    conf['default'] = dict(user = user, password = password)

    with open(GOSS_CREDENTIAL_PATH, 'w') as f:
        conf.write(f)
        click.secho('Success!', fg='cyan')

@run.command()
@click.option('--repo', '-r', is_flag = True, help = 'To create repository')
@click.option('--name', '-n', required = True, help = 'Create object name')
@click.option('--orga', '-o', help = 'If want create organization repository. Is required')
@click.pass_context
def create(ctx, repo, name, orga):
    g = ctx.obj
    if not g:
        click.echo('Seems you c')
        ctx.exit()

    if repo:
        click.echo('Begin create repository')
        click.echo('{}\t: https://github.com/{}/{}'.format(
            utils.make_progress_msg('Url'), name, name))
        click.secho('Waiting...', fg='yellow')
        if orga:
            flag, data = g.create_organization_repository(orga, name)
        else:
            flag, data = g.create_repository(name)
        if not flag:
            utils.print_error(data['message'])
            for err in data['errors']:
                utils.print_error('\t' + err['message'])
        else:
            utils.print_success()

@run.command()
@click.argument('name')
@click.argument('value')
@click.pass_context
def config(ctx, name, value):
    print(name, value)
    pass

if __name__ == "__main__":
    run()
    #  credentials()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
# Description:

import click

def print_success(msg='Success!'):
    click.secho(msg, fg='cyan')

def print_progress(msg='Waiting...'):
    click.secho(msg, fg='yellow')

def print_error(msg):
    click.secho(msg, fg='red')

def make_progress_msg(msg):
    return click.style(msg, fg = 'yellow')

def make_error_msg(msg):
    return click.style(msg, fg = 'red')



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
# Description:

import click
from goss import Github
from goss.app import utils
import configparser
import os
import pyperclip

github = None
owner = None
repo = None

GOSS_CONFIG_HOME = '{}/.config/goss'.format(os.getenv("HOME"))
GOSS_CONFIG_PATH = '{}/config'.format(GOSS_CONFIG_HOME)
GOSS_CREDENTIAL_PATH = '{}/credentials'.format(GOSS_CONFIG_HOME)

def init_git_config():
    '''初始化 git 信息'''
    #  if not os.path.exists(GOSS_CONFIG_HOME):
        #  os.mkdir(GOSS_CONFIG_HOME)
    credential_path = GOSS_CREDENTIAL_PATH

    config_path = GOSS_CONFIG_PATH

    if not os.path.exists(GOSS_CREDENTIAL_PATH):
        return None, ''

    credential = configparser.ConfigParser()
    credential.read(credential_path)
    default_cred = credential['default']

    g = Github(default_cred['user'], default_cred['password'])

    config = configparser.ConfigParser()
    config.read(config_path)
    author = config['user']
    g.set_author(author['name'], author['email'])

    return g

def generate_github():
    config_path = '{}/.config/gos/config'.format(os.getenv("HOME"))
    credential_path = '{}/.config/gos/credentials'.format(os.getenv("HOME"))

    credential = configparser.ConfigParser()
    credential.read(credential_path)
    default_cred = credential['default']

    global github
    if not github:
        github = Github(default_cred['user'], default_cred['password'])

    config = configparser.ConfigParser()
    config.read(config_path)
    author = config['user']
    github.set_author(author['name'], author['email'])

    global owner
    global repo
    owner = config['repository']['owner']
    repo = config['repository']['repo']

def create(filepath, path=None, repo=None, yes=False):
    '''
    创建文件
    '''
    if not path:
        path = os.path.basename(filepath)

    config = configparser.ConfigParser()
    config.read(GOSS_CONFIG_PATH)
    if not repo:
        repo = config['repository']['repo']
    click.echo('{}\t\t: {}'.format(click.style('Upload', fg='yellow'), filepath))
    click.echo('{}\t\t: https://github.com/{}/{}/{}'.format(
        click.style("To", fg='yellow'),
        owner, repo, path))
    download_url = 'https://raw.githubusercontent.com/{}/{}/master/{}'.format(
        owner, repo, path
    )
    pyperclip.copy(download_url)
    click.echo('{}\t: {}'.format(
        click.style('Download url', fg='yellow'),
        click.style(download_url, fg='blue'),
    ))
    click.echo('Now you can use it in content with {}. Wait upload success it will be avaible'.format(
        click.style('<Ctrl-v>', fg='blue')
    ))
    utils.print_progress()
    g = init_git_config()
    code, data = g.create_from_file(owner, repo, filepath, path)

    if code == 422 and data.get("message") == \
            'Invalid request.\n\n"sha" wasn\'t supplied.':
        if not yes:
            if not click.confirm('This file is exists. Do you want tu replace?'):
                return
        else:
            click.echo('{} {}'.format(
                utils.make_error_msg('Waring!'),
                utils.make_progress_msg('This file is exists. And now replace it.')
                    ))

        utils.print_progress()
        code, data = g.get_file(owner, repo, path)
        sha = data['sha']
        code, data = g.create_from_file(owner, repo, filepath, path, sha)

        if code != 200:
            utils.print_error(data['message'])
            return
    utils.print_success()

@click.command()
@click.option('-u', '--user', help='Github user')
#  @click.option('--owner', help='Github repository owner')
@click.option('--repo', '-r', help='Github repository name')
@click.option('--path', help='Github repository file path')
@click.option('--yes', '-y', is_flag = True, default = False, help='Is anwer yes?')
@click.argument('filepath', type=click.Path(exists=True))
#  @click.confirmation_option(prompt='Are you sure you want to drop the db?')
def run(filepath, user, path, repo, yes):
    '''
    Github Object Storage
    '''
    generate_github()

    create(filepath, path, repo, yes)
    pass

if __name__ == "__main__":
    run()
